/**
 * @file sparkhelperfactory.es
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.helpers.SPARKHelperFactory
 * @see net.sourceforge.spark.helpers.SPARKHelperDecorator
 * @see net.sourceforge.spark.widgets.SPARKFactory
 * @see net.sourceforge.spark.widgets.Container
 * @see net.sourceforge.spark.widgets.Atom
 * @see net.sourceforge.spark.widgets.Widget
 * @see net.sourceforge.spark.widgets.SPARK
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * afettes at users.sourceforge.net
 * http://spark.sourceforge.net
 * http://sourceforge.net/projects/spark/
 */

/**
 * You should not call this directly. Rather call SPARKFactory.init();
 * private SPARKHelperFactory();
 */
function SPARKHelperFactory()
{
    this.DECORATOR = null;
}

/**
 * Holds singleton instance.
 */
SPARKHelperFactory.factory = null;

/**
 * Initialize the factory and get the instance.
 */
SPARKHelperFactory.init = function()
{
    if( SPARKHelperFactory.factory == null )
    {
        // register all the widget types first
        SPARK.registerWidgetType( FlowLayout.REGEX );
        SPARK.registerWidgetType( Button.REGEX );
        SPARK.registerWidgetType( Textfield.REGEX );

        SPARKHelperFactory.factory = new SPARKHelperFactory();
    }

    // return the instance
    return( SPARKHelperFactory.factory );
}

/**
 * Set the decorator for the factory.
 * @param in_decorator The decorator to use.
 */
SPARKHelperFactory.prototype.setDecorator = function( in_decorator )
{
    this.DECORATOR = in_decorator;
};

/**
 * Create a Widget based on the given SVG node.
 * @param in_node
 */
SPARKHelperFactory.prototype.createWidget = function( in_node )
{
    var bIsWidget = false;
    var widget    = null;

    // Ensure this is an actual SVG element
    if( in_node != null && in_node.nodeType && in_node.nodeType == 1 )
    {
        var sClass = in_node.getAttribute( "class" );

        // See if this is a supported widget type
        if( SPARK.isWidget( sClass ) )
        {
            bIsWidget = true;
        }
    }

    if( bIsWidget )
    {
        // Create the different types of widgets
        // seperated into atom and container widgets
        if( sClass.match( Atom.REGEX ) )
        {
            widget = this.createAtom( in_node );
        }
        else if( sClass.match( Container.REGEX ) )
        {
            widget = this.createContainer( in_node );
        }

        if( widget != null && this.DECORATOR != null )
        {
            // decorate the widget
            this.DECORATOR.decorate( widget );
        }
    }

    return( widget );
};

/**
 * Create a Container based on the given SVG node.
 * @param in_node The Container-Contents construct that holds the
 *                SVG for the widgets it contains.
 * @param in_container The container that these widgets will be
 *                     contained by.
 */
SPARKHelperFactory.prototype.createContents = function( in_node, in_container )
{
    for( var i = 0; i < in_node.childNodes.length; i++ )
    {
        var node = in_node.childNodes.item(i);
        var widget = this.createWidget( node );

        if( widget != null )
        {
            in_container.addWidget( widget );
        }
    }
};

/**
 * Private - should not be called directly.
 * Create an Atom based on the given SVG node.
 * @param in_node The SVG Node to be used to create the Atom.
 * @return The completed Atom.  May return null.
 */
SPARKHelperFactory.prototype.createAtom = function( in_node )
{
    var widget = null;
    var sClass = in_node.getAttribute( "class" );

    if( sClass.match( Button.REGEX ) )
    {
        widget = new Button( in_node );
    }
    else if( sClass.match( Textfield.REGEX ) )
    {
        widget = new Textfield( in_node );
    }

    return( widget );
};

/**
 * Private - should not be called directly.
 * Create a Container based on the given SVG node.
 * @param in_node The SVG Node to be used to create the Container.
 * @return The completed Container.  May return null.
 */
SPARKHelperFactory.prototype.createContainer = function( in_node )
{
    var widget = null;
    var sClass = in_node.getAttribute( "class" );

	if( sClass.match( FlowLayout.REGEX ) )
	{
		widget = new FlowLayout( in_node );
	}

    return( widget );
};