/**
 * @file flowlayout.es
 *
 * Describes the FlowLayout Group Widget type from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.SPARK.Atom
 * @see net.sourceforge.spark.dom.SPARK.Container
 * @see net.sourceforge.spark.dom.SPARK.Observable
 * @see net.sourceforge.spark.dom.SPARK.Observer
 * @see net.sourceforge.spark.dom.SPARK.SPARK
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes@engr.uvic.ca
 *
 * Please forward all inquries to the following address:
 * Alastair Fettes
 * afettes at users.sourceforge.net
 */

/**
 * Create a new FlowLayout instance using an existing SVGElement
 * as the root.
 * @param in_node The root SVGElement.
 */
function FlowLayout( in_node )
{
	if( in_node != null )
	{
		// inheritance
        this.base = Container;
        this.base( in_node.getAttribute( "id" ), FlowLayout.CLASS );
        
        this.anchor = null;
        this.model  = null;

        // graphnode members
        this.anchor = null;
        
        this.width  = 200;
    
        for( var i = 0; i < in_node.childNodes.length; i++ )
        {
            var node = in_node.childNodes.item(i);
    
            if( node.nodeType == 1 )
            {
                if( node.nodeName == "desc" && this.anchor == null )
                {
                    this.anchor = node;
                }
                else if( node.nodeName == "metadata" && this.model == null )
                {
                    this.model = node;
                }
            }
        }
        
        this.updateLayout();
    }
}

/**
 * Regular expression defining the class attribute for a FlowLayout
 * SVGElement root element.
 */
FlowLayout.REGEX = /^SPARK container FlowLayout/;

/**
 * String expression defining the class attribute for a FlowLayout
 * SVGElement root element.
 */
FlowLayout.CLASS = "SPARK container FlowLayout";

/**
 * public class FlowLayout extends Container
 */
FlowLayout.prototype = new Container;

/**
 * Update the layout of the widget.
 */
FlowLayout.prototype.updateLayout = function()
{
    for( var i = 0; i < this.anchor.parentNode.childNodes.length; i++ )
    {
        var node = this.anchor.parentNode.childNodes.item(i);
    
        if( node.nodeType == 1 )
        {
            if( node.getAttribute( "class" ).match( Container.CONTENTS_REGEX ) )
            {
                // init any widgets
                var factory = SPARK.getFactory();
                factory.createContents( node, this );
            
                var x = 0;
                var y = 0;
                var totalW = 0;
                var totalH = 0;
                var maxH   = 0;
                
                for( var j = 0; j < node.childNodes.length; j++ )
                {
                    // add event handlers to all svg
                    var t_node = node.childNodes.item(j);
            
                    if( t_node.nodeType == 1 )
                    {
                        // clear its position first (if it had one)
                        t_node.setAttribute( "transform", ""  );
                        var bbox  = t_node.getBBox();
                        var currx = bbox.x;
                        var curry = bbox.y;
                        
                        var width  = bbox.width;
                        var height = bbox.height;
                        
                        // reset if we hit the end
                        if( (totalW + width) > this.width )
                        {
                            x = -currx;
                            y = totalH + maxH - curry;
            
                            totalH = totalH + maxH;
                            
                            totalW = width;                            
                            maxH   = height;
                        }
                        else
                        {
                            y = totalH - curry;
                            x = totalW - currx;
                            totalW += width;
                            
                            if( height > maxH )
                            {
                                maxH = height;
                            }
                        }
                        
                        var transform = "translate(" + x + "," + y + ")";
                        
                        t_node.setAttribute( "transform", transform  );
                    }
                }
            }
        }
    }
}

/**
 * Factory function for creating a new FlowLayout.
 * @param in_parent The parent SVGElement to this widget.
 * @return The new FlowLayout.
 */
FlowLayout.createWidget = function( in_parent )
{
    // to do
    return( null );
};