/**
 * @file textfield.es
 *
 * Describes the Textfield Widget type from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.spark.Atom
 * @see net.sourceforge.spark.dom.spark.Observable
 * @see net.sourceforge.spark.dom.spark.Widget
 * @see net.sourceforge.spark.dom.spark.CommandHolder
 *
 * Copyright (C) 2004  Christopher Lewis, Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * public class Textfield extends Atom
 *
 * @param in_node The SVG base node for the button widget.
 */
function Textfield( in_node )
{
	if( in_node != null )
	{
        this.anchor = null;
        this.model  = null;
        this.m_textID = "";
        this.m_rectID = "";
        this.m_cursor = null;
       
		// inheritance
        this.base = Atom;
        this.base( in_node.getAttribute( "id" ), Textfield.CLASS );
    
        // jogdial members with default values
        this.state      = "";
        
        // the starting index for selecting data. 
        // -1 refers to no selection chosen
        this.iSelectStart = -1;
    
        // local variables for initialization
        var bModelData = false;
        var strData    = "";
        
		for( var i = 0; i < in_node.childNodes.length; i++ )
		{
			var node = in_node.childNodes.item(i);

			if( node.nodeType == 1 )
			{
				if( this.anchor == null && node.nodeName == "desc" )
				{
					this.anchor = node;
				}
				else if( this.model == null && node.nodeName == "metadata" )
				{
					// the data for the text is held in the "data" attribute of
                    // the model
					this.model = node;
                    
                    for( var j = 0; j < node.childNodes.length; j++ )
                    {
                        if( node.childNodes.item(j).nodeType == 1 )
                        {
                            this.model = node.childNodes.item(j);
                            break;
                        }
                    }
                    
                    var data = this.model.getAttribute( "data" );
                    
                    if( data != "" )
                    {
                        bModelData = true;
                        strData    = data;
                    }
				}
				else if( node.getAttribute( "id" ) == this.ID + "-text" )
				{
					this.textID = node.getAttribute( "id" );
                }
                else if( node.getAttribute( "id" ) == this.ID + "-rect" )
				{
					this.rectID = node.getAttribute( "id" );
                }
			}
		}
	}
    
    if( bModelData )
    {
        var SVGDoc  = this.anchor.ownerDocument;
        var txtbox  = SVGDoc.getElementById( this.textID );
        
        var setPrespan = false;
        
        for( var i = 0; i < txtbox.childNodes.length; i++ )
        {
            if( txtbox.childNodes.item(i).nodeType == 1 )
            {
                if( !setPrespan )
                {
                    setPrespan = true;
                    
                    var prespan = txtbox.childNodes.item( i );
                    var predata = prespan.getFirstChild();
                    if( predata == null )
                    {
                        predata = SVGDoc.createTextNode( "" );
                        prespan.appendChild( predata );
                    }
                }
                else
                {
                    var bufspan = txtbox.childNodes.item( i );
                    var bufdata = bufspan.getFirstChild();
                    if( bufdata == null )
                    {
                        bufdata = SVGDoc.createTextNode( "" );
                        bufspan.appendChild( bufdata );
                    }
                }
            }
        }

        if( predata != null )
        {
            predata.data = strData;
        }
        
        if( bufdata != null )
        {
            bufdata.data = "";
        }
        
        this.updateState();
    }
    else if( this.textID != null )
    {
        var SVGDoc  = this.anchor.ownerDocument;
        var txtbox  = SVGDoc.getElementById( this.textID );
        
        var setPrespan = false;
        var passedCursor = false;
        
        for( var i = 0; i < txtbox.childNodes.length; i++ )
        {
            if( txtbox.childNodes.item(i).nodeType == 1 )
            {
                if( !setPrespan )
                {
                    setPrespan = true;
                    
                    var prespan = txtbox.childNodes.item( i );
                    var predata = prespan.getFirstChild();
                    if( predata == null )
                    {
                        predata = SVGDoc.createTextNode( "" );
                        prespan.appendChild( predata );
                    }
                }
                else if( !passedCursor )
                {
                    passedCursor = true;
                }
                else
                {
                    var bufspan = txtbox.childNodes.item( i );
                    var bufdata = bufspan.getFirstChild();
                    if( bufdata == null )
                    {
                        bufdata = SVGDoc.createTextNode( "" );
                        bufspan.appendChild( bufdata );
                    }
                }
            }
        }
        
        this.updateState();
    }
    else
    {
        // to do
    }
    
    this.anchor.parentNode.addEventListener( "click", this, false );
    this.loseFocus();
}

/**
 * public class Textfield extends SPARK implements Observable
 */
Textfield.prototype = new Atom;

/**
 * Regular expression defining the class attribute for a Textfield
 * SVGElement root element.
 */
Textfield.REGEX = /^SPARK atom Textfield/;

/**
 * Regular expression defining the class attribute for a Textfield
 * SVGElement root element.
 */
Textfield.CLASS = "SPARK atom Textfield";

/**
 * Override the normal behaviour for the setState function.
 * We always want to store the new state into model.
 */
Textfield.prototype.setState = function( in_state )
{
    this.model.setAttribute( "data", in_state );
    this.state = in_state;
   
    // notify anyone watching
    this.notifyObservers();
    
    // we also want to run any commands that are available
    this.runCommands();
};

Textfield.prototype.loseFocus = function()
{
    if( SPARK.KEYBOARD_FOCUS == this )
    {
        SPARK.releaseKeyboardFocus(this);
        var SVGDoc  = this.anchor.ownerDocument;
        var txtbox  = SVGDoc.getElementById( this.textID );
        this.cursor = txtbox.childNodes.item( 1 );
        txtbox.removeChild( this.cursor );
    }
};

Textfield.prototype.gainFocus = function()
{
    if( SPARK.KEYBOARD_FOCUS != this )
    {
        SPARK.requestKeyboardFocus(this);
        var SVGDoc = this.anchor.ownerDocument;
        var txtbox = SVGDoc.getElementById( this.textID );
        var insert = txtbox.childNodes.item( 1 );
        txtbox.insertBefore( this.cursor, insert );
        this.cursor = null;
    }
};

/**
 * Call this function when a state change has occured and to mirror this
 * change to the text fields model.
 */
Textfield.prototype.updateState = function()
{
	var SVGDoc  = this.anchor.ownerDocument;
	var txtbox  = SVGDoc.getElementById( this.textID );
    
    var prespan = null;
    var bufspan = null;
    
    setPrespan = false;
    passedCursor = false;
    
    for( var i = 0; i < txtbox.childNodes.length; i++ )
    {
        if( txtbox.childNodes.item(i).nodeType == 1 )
        {
            if( !setPrespan )
            {
                setPrespan = true;
                
                prespan = txtbox.childNodes.item( i );
            }
            else if( !passedCursor )
            {
                passedCursor = true;
            }
            else
            {
                bufspan = txtbox.childNodes.item( i );
            }
        }
    }

    if( prespan != null && bufspan != null )
    {
        var predata = prespan.getFirstChild();
        var bufdata = bufspan.getFirstChild();
        
        var rect    = SVGDoc.getElementById( this.rectID );
        rect.setAttribute( "width", 0 );
        rect.setAttribute( "height", 0 );
        
        var width  = 0;
        var height = 0;
        
        if( width < parseInt( this.model.getAttribute( "minw" ) ) )
        {
            width = this.model.getAttribute( "minw" );
        }
        else if( width > parseInt( this.model.getAttribute( "maxw" ) ) )
        {
            width = this.model.getAttribute( "maxw" );
        }
        
        if( height < parseInt( this.model.getAttribute( "minh" ) ) )
        {
            height = this.model.getAttribute( "minh" );
        }
        else if( height > parseInt( this.model.getAttribute( "maxh" ) ) )
        {
            height = this.model.getAttribute( "maxh" );
        }
        
        rect.setAttribute( "width", width*1.2 );
        rect.setAttribute( "height", height*1.5 );
        
        this.setState( predata.data + bufdata.data );
    }
};

/*
 * ********************
 * START event handling
 * ********************
 */

Textfield.prototype.click = function( in_evt )
{
    if( SPARK.requestMouseFocus(this) )
    {
        this.gainFocus();    
        this.iSelectStart = -1;
        this.setSelected();
        
        var SVGDoc = this.anchor.ownerDocument;
        var tbox   = SVGDoc.getElementById( this.textID );
        
        if( tbox.getNumberOfChars() > 0 )
        {
            var xPos    = in_evt.clientX;
            var yPos    = in_evt.clientY;
            var point   = getNodeCoordinates( this.anchor.parentNode, xPos, yPos );         
            var charPos = this.getCharAt(point.x);
            this.cursor_place( charPos );
        }
        
        in_evt.stopPropagation();
        
        SPARK.releaseMouseFocus(this);
    }
};

Textfield.prototype.mousedown = function( in_evt )
{
    if( SPARK.requestMouseFocus(this) )
    {
        this.gainFocus();
        this.iSelectStart = -1;
        this.setSelected();
        
        var SVGDoc = this.anchor.ownerDocument;
        var tbox   = SVGDoc.getElementById( this.textID );
        
        if( tbox.getNumberOfChars() > 0 )
        {
            var xPos    = in_evt.clientX;
            var yPos    = in_evt.clientY;
            var point   = getNodeCoordinates( this.anchor.parentNode, xPos, yPos );         
            var charPos = this.getCharAt(point.x);
            this.iSelectStart = charPos;
            this.cursor_place( charPos );
        }
        
        in_evt.stopPropagation();
    }
};

Textfield.prototype.mouseup = function( in_evt )
{
    if( SPARK.releaseMouseFocus(this) )
    {
        /* this.gainFocus();
        
        var xPos  = in_evt.clientX;
        var yPos  = in_evt.clientY;
        var point = getNodeCoordinates( this.anchor.parentNode, xPos, yPos );
        
        
        
        in_evt.stopPropagation();*/
    }
};

Textfield.prototype.getCharAt = function( in_xPos )
{
    var SVGDoc  = this.anchor.ownerDocument;
	var tbox    = SVGDoc.getElementById( this.textID );
    var charPos = tbox.getNumberOfChars()-1;
    
    // could use some 
    for( var i = 0; i < tbox.getNumberOfChars(); i++ )
    {
        var stop = tbox.getEndPositionOfChar(i).x;
        
        if( in_xPos <= stop )
        {
            var start  = tbox.getStartPositionOfChar(i).x;        
            var middle = (start + stop)/2;
            
            if( in_xPos <= middle )
            {
                charPos = i;
            }
            else
            {
                charPos = i+1;
            }
            break;
        }
    }
    
    return( charPos );
};

Textfield.prototype.keypress = function( in_evt )
{
    if( SPARK.KEYBOARD_FOCUS == this )
    {
        var key      = in_evt.getCharCode();
        var char     = String.fromCharCode(key);
        var bCtrlKey = in_evt.ctrlKey;
        var bShftKey = in_evt.shiftKey;
        
        if( !bShftKey )
        {
            if( this.iSelectStart > -1 )
            {
                var SVGDoc = this.anchor.ownerDocument;
                var txtbox = SVGDoc.getElementById( this.textID );
                var tspan  = txtbox.childNodes.item( 0 );
                var data   = tspan.getFirstChild();
                
                var iSelectEnd = data.getLength();
                
                if( iSelectEnd < this.iSelectStart )
                {
                    alert( iSelectEnd + " to " + this.iSelectStart );
                    this.delete_range( iSelectEnd, this.iSelectStart - iSelectEnd );
                }
                else
                {
                    alert( iSelectEnd + " to " + this.iSelectStart );
                    this.delete_range( this.iSelectStart, iSelectEnd - this.iSelectStart );
                }
                
                this.iSelectStart = -1;
                this.updateState();
                this.setSelected();
                
                if( key == Textfield.CHARCODE_BACKSPACE )
                {
                    in_evt.stopPropagation();
                    return;
                }
            }
            
            this.iSelectStart = -1;
        }
        else if( this.iSelectStart == -1 )
        {
            this.setSelectStart();
        }
        
        if( char == "5" )
        {
            //alert( serializeNode(this.anchor.parentNode) );
        }
        
        if( key >= Textfield.CHARCODE_START_ALPHANUMERIC && 
            key <= Textfield.CHARCODE_END_ALPHANUMERIC )
        {
            this.addText( char );
        }
        else
        {
            switch( key )
            {
                case Textfield.CHARCODE_BACKSPACE:
                    this.backspace();
                    break;
                default:
                    break;
            }
        }
        
        this.setSelected();
    }
};

Textfield.prototype.keyup = function( in_evt )
{
    if( SPARK.KEYBOARD_FOCUS == this )
    {
        var key      = in_evt.getKeyCode();
        var bShftKey = in_evt.shiftKey;
        
        if( bShftKey && this.iSelectStart == -1 )
        {
            this.setSelectStart();
        }
    }
};

Textfield.prototype.keydown = function( in_evt )
{
    if( SPARK.KEYBOARD_FOCUS == this )
    {
        var key = in_evt.getKeyCode();
        var bCtrlKey = in_evt.ctrlKey;
        var bShftKey = in_evt.shiftKey;
        
        if( !bShftKey )
        {
            if( this.iSelectStart > -1 )
            {
                var SVGDoc = this.anchor.ownerDocument;
                var txtbox = SVGDoc.getElementById( this.textID );
                var tspan  = txtbox.childNodes.item( 0 );
                var data   = tspan.getFirstChild();
                
                var iSelectEnd = tspan.getNumberOfChars();
                
                if( iSelectEnd < this.iSelectStart )
                {
                    this.delete_range( iSelectEnd, this.iSelectStart - iSelectEnd );
                }
                else
                {
                    this.delete_range( this.iSelectStart, iSelectEnd - this.iSelectStart );
                }
                
                this.iSelectStart = -1;
                this.updateState();
                this.setSelected();
                
                if( key == Textfield.KEYCODE_DELETE )
                {
                    in_evt.stopPropagation();
                    return;
                }
            }
            
            this.iSelectStart = -1;
        }
        else if( this.iSelectStart == -1 )
        {
            this.setSelectStart();
        }
        
        switch( key )
        {
            case Textfield.KEYCODE_LEFT:
                this.cursor_left(bCtrlKey);
                break;
            case Textfield.KEYCODE_RIGHT:
                this.cursor_right(bCtrlKey);
                break;
            case Textfield.KEYCODE_HOME:
                this.cursor_place( 0 );
                break;
            case Textfield.KEYCODE_END:
                var SVGDoc = this.anchor.ownerDocument;
                var tbox  = SVGDoc.getElementById( this.textID );
                this.cursor_place( tbox.getNumberOfChars() );
                break;
            case Textfield.KEYCODE_TAB:     // won't get caught my MSIE
                this.addText( "    " );
                break;
            case Textfield.KEYCODE_ESCAPE:
                this.loseFocus();
                break;
            case Textfield.KEYCODE_DELETE:
                this.del();
                break;
            default:
                break;
        }
        
        this.setSelected();
        
        evt.stopPropagation();
    }
};
 
/**
 *
 * a = 65, z = 90
 */
// charCode for evt.getCharCode()
Textfield.CHARCODE_A                    = 65;    // for ctrl-c
Textfield.CHARCODE_C                    = 67;    // for ctrl-c
Textfield.CHARCODE_V                    = 86;    // for ctrl-c
Textfield.CHARCODE_X                    = 88;    // for ctrl-c
Textfield.CHARCODE_BACKSPACE            = 8; 
Textfield.CHARCODE_START_ALPHANUMERIC   = 32;    // first alpha-numeric - ' '
Textfield.CHARCODE_END_ALPHANUMERIC     = 126;   // last alpha-numeric - '~'

// keyCode for evt.getKeyCode()
Textfield.KEYCODE_CTRL_A                = 1;   // select all
Textfield.KEYCODE_TAB                   = 9;   // MSIE won't let you catch this
Textfield.KEYCODE_CTRL_V                = 22;  // paste
Textfield.KEYCODE_CTRL_X                = 24;  // cut and copy
Textfield.KEYCODE_ESCAPE                = 27;
Textfield.KEYCODE_END                   = 35;
Textfield.KEYCODE_HOME                  = 36;
Textfield.KEYCODE_LEFT                  = 37;  // directionals
Textfield.KEYCODE_UP                    = 38;  
Textfield.KEYCODE_RIGHT                 = 39;
Textfield.KEYCODE_DOWN                  = 40;
Textfield.KEYCODE_DELETE                = 127;

/*
 * ********************
 * START text insertion
 * ********************
 */

/**
 * Add the characters to the given line.
 * @param in_text The text.
 * @param in_line The line.
 */
Textfield.prototype.addText = function( in_text )
{
    var SVGDoc = this.anchor.ownerDocument;
    var tbox   = SVGDoc.getElementById( this.textID );
    var width  = tbox.getComputedTextLength();
    var maxw   = parseInt( this.model.getAttribute( "maxw" ) );

    if( maxw != "NaN" && width < maxw )
    {
        var SVGDoc = this.anchor.ownerDocument;
        var tbox  = SVGDoc.getElementById( this.textID );
        var tspan = tbox.childNodes.item( 0 );
        var data  = tspan.getFirstChild();

        data.appendData( in_text );

        this.updateState();
    }
};

/*
 * ******************
 * START text removal
 * ******************
 */

/**
 * Delete the first char after the cursor.
 */
Textfield.prototype.del = function()
{
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
	var tspan = tbox.childNodes.item( 2 );
	var data  = tspan.getFirstChild();
    
    if( data.getLength() > 0 )
    {
        data.deleteData( 0, 1 );
        this.updateState();
    }
};

/**
 * Delete the first char before the cursor.
 */
Textfield.prototype.backspace = function()
{
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
	var tspan = tbox.childNodes.item( 0 );
	var data  = tspan.getFirstChild();

    if( data.getLength() > 0 )
    {
        data.deleteData( data.getLength()-1, 1 );
        this.updateState();
    }
};

/**
 * Delete a substring 
 * NOTE: NOT TESTED!
 *
 * @param in_start The start position (>=1).
 * @param in_length The number of chars to delete.
 * @return The deleted substring.
 */
Textfield.prototype.delete_range = function( in_start, in_length )
{
    // will hold the text that was deleted
    var deleted;
    
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
    
	var prespan = tbox.childNodes.item( 0 );
	var predata = prespan.getFirstChild();
  
	var bufspan = tbox.childNodes.item( 2 );
	var bufdata = bufspan.getFirstChild();
    
    if( in_start > predata.getLength()-1 )
    {
        deleted = bufdata.substringData( 0, in_length );
        bufdata.deleteData( 0, in_length );
        this.updateState();
    }
    else
    {
        deleted = predata.substringData( in_start, predata.getLength()-in_start );
        predata.deleteData( in_start, predata.getLength()-in_start );
        this.updateState();
    }
    
    return( deleted );
};

/*
 * **********************
 * START cursor functions
 * **********************
 */

 /**
  * Set the start of text selection.
  * NOTE: NOT TESTED!!!
  * @param in_bCtrlKey if the ctrl key was held while doing this.
  */
Textfield.prototype.setSelectStart = function( in_bCtrlKey )
{
	var SVGDoc  = this.anchor.ownerDocument;
	var tbox    = SVGDoc.getElementById( this.textID );
	var prespan = tbox.childNodes.item( 0 );
    
    this.iSelectStart = prespan.getNumberOfChars();
};

 /**
  * Set the selected text to be selected.
  * NOTE: NOT TESTED!!!
  */
Textfield.prototype.setSelected = function()
{
    var SVGDoc  = this.anchor.ownerDocument;
	var tbox    = SVGDoc.getElementById( this.textID );
	var prespan = tbox.childNodes.item( 0 );
    
    var iSelectEnd = prespan.getNumberOfChars();
    
    if( this.iSelectStart < 0 )
    {
        tbox.selectSubString( 0, 0 );
    }
    else if( iSelectEnd < this.iSelectStart )
    {
        tbox.selectSubString( iSelectEnd, this.iSelectStart - iSelectEnd + 1 );
    }
    else
    {
        tbox.selectSubString( this.iSelectStart,  iSelectEnd - this.iSelectStart );
    }
}
 
/**
 * Move the cursor left.
 */
Textfield.prototype.cursor_left = function( in_bCtrlKey )
{
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
    
	var prespan = tbox.childNodes.item( 0 );
	var predata = prespan.getFirstChild();
  
	var bufspan = tbox.childNodes.item( 2 );
	var bufdata = bufspan.getFirstChild();
    
    if( predata.getLength() < 1 )
    {
        // do nothing, already at start
    }
    else if( in_bCtrlKey )
    {
        // this will move to the start of the previous/current word
        var moveChars = "";
        
        if( predata.getLength() > 0 && 
            predata.substringData( predata.getLength()-1, 1 ) == " " )
        {
            moveChars = " ";
            predata.deleteData( predata.getLength()-1, 1 );
        }
        
        while( predata.getLength() > 0 && 
               predata.substringData( predata.getLength()-1, 1 ) != " " )
        {
            moveChars = predata.substringData( predata.getLength()-1, 1 ) + moveChars;
            predata.deleteData( predata.getLength()-1, 1 );
        }
        
        bufdata.insertData( 0, moveChars );
    }
    else
    {
        var moveChar = predata.substringData( predata.getLength()-1, 1 );
        predata.deleteData( predata.getLength()-1, 1 );
        bufdata.insertData( 0, moveChar );
    }
};

/**
 * Move the cursor right.
 */
Textfield.prototype.cursor_right = function( in_bCtrlKey )
{
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
    
	var prespan = tbox.childNodes.item( 0 );
	var predata = prespan.getFirstChild();
  
	var bufspan = tbox.childNodes.item( 2 );
	var bufdata = bufspan.getFirstChild();
    
    if( bufdata.getLength() < 1 )
    {
        // do nothing, already at end
    }
    else if( in_bCtrlKey )
    {
        // this will move to the start of the next word
        var moveChars = "";
        
        while( bufdata.getLength() > 0 && bufdata.substringData( 0, 1 ) != " " )
        {
            moveChars += bufdata.substringData( 0, 1 );
            bufdata.deleteData( 0, 1 );
        }
        
        if( bufdata.getLength() > 0 && bufdata.substringData( 0, 1 ) == " " )
        {
            moveChars += " ";
            bufdata.deleteData( 0, 1 );
        }
        
        predata.appendData( moveChars );
    }
    else
    {
        var moveChar = bufdata.substringData( 0, 1 );
        bufdata.deleteData( 0, 1 );
        predata.appendData( moveChar );
    }
};

/**
 * Move the cursor to the given position.
 * @param in_pos The text position to set the cursor to.
 */
Textfield.prototype.cursor_place = function( in_pos )
{
	var SVGDoc = this.anchor.ownerDocument;
	var tbox  = SVGDoc.getElementById( this.textID );
    
	var prespan = tbox.childNodes.item( 0 );
	var predata = prespan.getFirstChild();
  
	var bufspan = tbox.childNodes.item( 2 );
	var bufdata = bufspan.getFirstChild();

    predata.data = predata.data + bufdata.data;
    
    if( in_pos < 0 )
    {
        in_pos = 0;
    }
    else if( in_pos > predata.getLength() )
    {
        in_pos = predata.getLength();
    }
    
    bufdata.data = predata.substringData( in_pos, predata.getLength()-in_pos );
    predata.deleteData( in_pos, predata.getLength()-in_pos );
};

/*
 * **************
 * MISC FUNCTIONS
 * **************
 */

/**
 * Interface member.
 * @param in_observable
 */
Textfield.prototype.update = function( in_observable )
{
	// implement
};

/*
 * Static functions.
 *
Textfield.createTspan = function()
{
	return( SVGDoc.createElementNS( "tspan" ) );
};

Textfield.createBuffer = function( in_id )
{
	var tspan = Textfield.createTspan( in_id );
	tspan.setAttribute( "id", in_id + "-buffer" );
};

Textfield.createCursor = function( in_id )
{
	var tspan = Textfield.createTspan( in_id );
	tspan.setAttribute( "id", in_id + "-cursor" );


	return( tspan );
};*/