/**
 * @file container.es
 *
 * Describes the Container type from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.dom.spark.Atom
 * @see net.sourceforge.spark.dom.spark.Container
 * @see net.sourceforge.spark.dom.spark.Observable
 * @see net.sourceforge.spark.dom.spark.Observer
 * @see net.sourceforge.spark.dom.spark.Widget
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * A container is a widget that may contain other widgets.
 * @param id The id of the widget.
 * @param type The type of the widget.
 */
function Container( id, type )
{
    if( id && type )
    {
        // extends Widget
        this.base = Widget;
        this.base( id, type );
    }

    // implements Observerable
    this.m_vpObservers = new Array();

    // contains its contents
    this.m_vpWidgets   = new Array();
}

// *** ADDED FOR SCRIPTING ***
Container.prototype      = new Widget;
Container.REGEX     	 = /^SPARK container/;
Container.CONTENTS_REGEX = /^SPARK contents/;

/**
 * Add a widget to this container.
 * @param in_widget The widget to be added.
 */
Container.prototype.addWidget = function( in_widget )
{
    var bAdd = true;

    for( var i = 0; i < this.m_vpWidgets.length; i++ )
    {
        if( this.m_vpWidgets[i] == in_widget )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        this.m_vpWidgets.push( in_widget );
        in_widget.attach( this );
    }
};

/**
 * Remove a widget from this container.
 * @param in_widget The widget to be removed.
 */
Container.prototype.removeWidget = function( in_widget )
{
    var bFound = false;

    for( var i = 0; i < this.m_vpWidgets.length; i++ )
    {
        if( this.m_vpWidgets[i] == in_widget && !bFound )
        {
            this.m_vpWidgets[i] = null;
            bFound = true;

            if( (i+1) < this.m_vpWidgets.length )
            {
                this.m_vpWidgets[i]   = this.m_vpWidgets[i+1];
                this.m_vpWidgets[i+1] = null;
            }
        }
        else if( bFound && (i+1) < this.m_vpWidgets.length )
        {
            this.m_vpWidgets[i]   = this.m_vpWidgets[i+1];
            this.m_vpWidgets[i+1] = null;
        }
    }
};

/**
 * The observer to be attached.  It will now be
 * observering this subject.
 * @param in_observer The observer to add.
 */
Container.prototype.attach = function( in_observer )
{
    var bAdd = true;

    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
        if( this.m_vpObservers[i] == in_observer )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        this.m_vpObservers.push( in_observer );
    }
};

/**
 * The observer to be detached.  It will no longer
 * be observering this subject.
 * @param in_observer The observer to remove.
 */
Container.prototype.detach = function( in_observer )
{
    var bFound = false;

    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
        if( !bFound && this.m_vpObservers[i] == in_observer )
        {
            this.m_vpObservers[i] = null;
            bFound = true;

            if( (i+1) < this.m_vpObservers.length )
            {
                this.m_vpObservers[i]   = this.m_vpObservers[i+1];
                this.m_vpObservers[i+1] = null;
            }
        }
        else if( bFound && (i+1) < this.m_vpObservers.length )
        {
            this.m_vpObservers[i]   = this.m_vpObservers[i+1];
            this.m_vpObservers[i+1] = null;
        }
    }
};

/**
 * Notify all observers of the state change.
 */
Container.prototype.notifyObservers = function()
{
    for( var i = 0; i < this.m_vpObservers.length; i++ )
    {
		this.m_vpObservers[i].update( this );
    }
};

/**
 * A change occurred in one of this containers subjects.
 */
Container.prototype.update = function( in_subject )
{
    this.notifyObservers();
};