/**
 * @file spark.es
 *
 * Describes the SPARK class from the SPARK framework.
 *
 * @author Alastair Fettes
 *
 * @see net.sourceforge.spark.widgets.SPARK
 *
 * Copyright (C) 2004  Alastair Fettes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Alastair Fettes
 * http://spark.sourceforge.net
 * afettes at users.sourceforge.net
 */

/**
 * This is a static/singleton class and therefore the
 * constructor should not be invoked.
 */
function SPARK(){};

/**
 * The widget that currently has the mouse focus.
 */
SPARK.MOUSE_FOCUS = null;

/**
 * The widget that currently has the keyboard focus.
 */
SPARK.KEYBOARD_FOCUS = null;

/**
 * A set of widgets that have been registered globally.
 */
SPARK.REGISTERED_TYPES = new Array();

/**
 * A set of widgets that have been registered globally.
 */
SPARK.REGISTERED_WIDGETS = new Array();

/**
 * The factory object used to create widgets in the framework.
 */
SPARK.FACTORY = null;

/**
 * Let a widget request mouse focus.  If another widget
 * already has mouse focus this action will fail.
 * @param in_widget The widget requesting keyboard focus.
 * @return If the widget successfully retrieved mouse focus.
 */
SPARK.requestMouseFocus = function( in_widget )
{
    var bSuccess = false;

    if( SPARK.MOUSE_FOCUS == null )
    {
        SPARK.MOUSE_FOCUS = in_widget;
        bSuccess = true;
    }
    else if( SPARK.MOUSE_FOCUS == in_widget )
    {
        bSuccess = true;
    }

    return( bSuccess );
};

/**
 * Release the widget from global mouse focus.
 * @param in_widget The widget to be removed from focus.
 * @return True if the widget had the mouse focus and it
 *         was removed successfully.
 */
SPARK.releaseMouseFocus = function( in_widget )
{
    var bSuccess = false;
    
    if( SPARK.MOUSE_FOCUS == null )
    {
        bSuccess = true;
    }
    else if( SPARK.MOUSE_FOCUS == in_widget )
    {
        SPARK.MOUSE_FOCUS = null;
        bSuccess = true;
    }
    
    return( bSuccess );
};

/**
 * This will distribute the given event to the appropriate
 * widget that has mouse focus.
 */
SPARK.handleMouseEvent = function( evt )
{
    if( SPARK.MOUSE_FOCUS != null )
    {
        SPARK.MOUSE_FOCUS.handleEvent( evt );
    }
};

/**
 * Let a widget request keyboard focus.  Keyboard Focus
 * will always be granted.
 * @param in_widget The widget requesting keyboard focus.
 */
SPARK.requestKeyboardFocus = function( in_widget )
{
    SPARK.KEYBOARD_FOCUS = in_widget;
};

/**
 * Release the widget from global keyboard focus only if it
 * has current keyboard focus.
 * @param in_widget The widget.
 */
SPARK.releaseKeyboardFocus = function( in_widget )
{
    if( SPARK.KEYBOARD_FOCUS == in_widget )
    {
        SPARK.KEYBOARD_FOCUS = null;
    }
};

/**
 * This will distribute the given event to the appropriate
 * widget that has keyboard focus.
 */
SPARK.handleKeyboardEvent = function( evt )
{
    if( SPARK.KEYBOARD_FOCUS != null )
    {
        SPARK.KEYBOARD_FOCUS.handleEvent( evt );
    }
};

/**
 * Add a widget type to the allowed type list.
 * @param in_type The type to be added.
 */
SPARK.registerWidgetType = function( in_type )
{
    var bAdd = true;

    for( var i = 0; i < SPARK.REGISTERED_TYPES.length; i++ )
    {
        if( SPARK.REGISTERED_TYPES[i] == in_type )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        SPARK.REGISTERED_TYPES.push( in_type );
    }
};

/**
 * Remove a widget type frome the allowed type list.
 * @param in_type The type to be removed.
 */
SPARK.deregisterWidgetType = function( in_type )
{
    var bFound = false;

    for( var i = 0; i < SPARK.REGISTERED_TYPES.length; i++ )
    {
        if( SPARK.REGISTERED_TYPES[i] == in_type && !bFound )
        {
            SPARK.REGISTERED_TYPES[i] = null;
            bFound = true;
        }

        if( bFound && (i+1) < SPARK.REGISTERED_TYPES.length )
        {
            SPARK.REGISTERED_TYPES[i] = SPARK.REGISTERED_TYPES[i+1];
        }
    }
};

/**
 * Test to see if a given string defines a currently
 * registered widget type.
 * @param in_type The type to test.
 * @return True if that type has been registered.
 */
SPARK.isWidget = function( in_name )
{
    var bRegistered = false;

    for( var i = 0; i < SPARK.REGISTERED_TYPES.length; i++ )
    {
        if( in_name.match( SPARK.REGISTERED_TYPES[i] ) )
        {
            bRegistered = true;
            break;
        }
    }

    return( bRegistered );
};

/**
 * Add a widget to the global registration list.
 * @param in_widget The widget to be added.
 */
SPARK.registerWidget = function( in_widget )
{
    var bAdd = true;

    for( var i = 0; i < SPARK.REGISTERED_WIDGETS.length; i++ )
    {
        if( SPARK.REGISTERED_WIDGETS[i] == in_widget )
        {
            bAdd = false;
            break;
        }
    }

    if( bAdd )
    {
        SPARK.REGISTERED_WIDGETS.push( in_widget );
    }
};

/**
 * Remove a widget from the global registration list.
 * @param in_widget The widget to be removed.
 */
SPARK.deregisterWidget = function( in_widget )
{
    var bFound = false;

    for( var i = 0; i < SPARK.REGISTERED_WIDGETS.length; i++ )
    {
        if( SPARK.REGISTERED_WIDGETS[i] == in_widget && !bFound )
        {
            SPARK.REGISTERED_WIDGETS[i] = null;
            bFound = true;
        }

        if( bFound && (i+1) < SPARK.REGISTERED_WIDGETS.length )
        {
            SPARK.REGISTERED_WIDGETS[i] = SPARK.REGISTERED_WIDGETS[i+1];
        }
    }
};


/**
 * Remove a widget from the global registration list.
 * @param in_widget The widget to be removed.
 */
SPARK.getRegisteredWidget = function( in_id )
{
    var widget = null;

    for( var i = 0; i < SPARK.REGISTERED_WIDGETS.length; i++ )
    {
        if( SPARK.REGISTERED_WIDGETS[i].ID == in_id )
        {
            widget = SPARK.REGISTERED_WIDGETS[i];
            break;
        }
    }
    
    return( widget );
};

/**
 * Set the factory to be used to create new widget instances.
 * @param in_factory The factory.
 */
SPARK.setFactory = function( in_factory )
{
    SPARK.FACTORY = in_factory;
};

/**
 * Get the factory to be used to create new widget instances.
 * @param in_factory The factory.
 */
SPARK.getFactory = function()
{
    return( SPARK.FACTORY );
};